@@ -21,12 +21,21 @@
 #include <private/qqmltypedata_p.h>
 #include <private/inlinecomponentutils_p.h>
 #include <private/qqmlsourcecoordinate_p.h>
+#include <private/qqmlsignalnames_p.h>
+
+#include <QtCore/qloggingcategory.h>
+#include <QtCore/qscopedvaluerollback.h>
+
+#if QT_CONFIG(regularexpression)
+#include <QtCore/qregularexpression.h>
+#endif
 
-#include <QScopedValueRollback>
 #include <vector>
 
 QT_BEGIN_NAMESPACE
 
+Q_DECLARE_LOGGING_CATEGORY(invalidOverride);
+
 inline QQmlError qQmlCompileError(const QV4::CompiledData::Location &location,
                                                   const QString &description)
 {
@@ -66,12 +75,101 @@ struct QQmlPropertyCacheCreatorBase
 public:
     static QAtomicInt Q_AUTOTEST_EXPORT classIndexCounter;
 
-    static QMetaType metaTypeForPropertyType(QV4::CompiledData::CommonType type);
-    static QMetaType listTypeForPropertyType(QV4::CompiledData::CommonType type);
+    static QMetaType metaTypeForPropertyType(QV4::CompiledData::CommonType type)
+    {
+        switch (type) {
+        case QV4::CompiledData::CommonType::Void:     return QMetaType();
+        case QV4::CompiledData::CommonType::Var:      return QMetaType::fromType<QVariant>();
+        case QV4::CompiledData::CommonType::Int:      return QMetaType::fromType<int>();
+        case QV4::CompiledData::CommonType::Bool:     return QMetaType::fromType<bool>();
+        case QV4::CompiledData::CommonType::Real:     return QMetaType::fromType<qreal>();
+        case QV4::CompiledData::CommonType::String:   return QMetaType::fromType<QString>();
+        case QV4::CompiledData::CommonType::Url:      return QMetaType::fromType<QUrl>();
+        case QV4::CompiledData::CommonType::Time:     return QMetaType::fromType<QTime>();
+        case QV4::CompiledData::CommonType::Date:     return QMetaType::fromType<QDate>();
+        case QV4::CompiledData::CommonType::DateTime: return QMetaType::fromType<QDateTime>();
+#if QT_CONFIG(regularexpression)
+        case QV4::CompiledData::CommonType::RegExp:   return QMetaType::fromType<QRegularExpression>();
+#else
+        case QV4::CompiledData::CommonType::RegExp:   return QMetaType();
+#endif
+        case QV4::CompiledData::CommonType::Rect:     return QMetaType::fromType<QRectF>();
+        case QV4::CompiledData::CommonType::Point:    return QMetaType::fromType<QPointF>();
+        case QV4::CompiledData::CommonType::Size:     return QMetaType::fromType<QSizeF>();
+        case QV4::CompiledData::CommonType::Invalid:  break;
+        };
+        return QMetaType {};
+    }
 
+    static QMetaType listTypeForPropertyType(QV4::CompiledData::CommonType type)
+    {
+        switch (type) {
+        case QV4::CompiledData::CommonType::Void:     return QMetaType();
+        case QV4::CompiledData::CommonType::Var:      return QMetaType::fromType<QList<QVariant>>();
+        case QV4::CompiledData::CommonType::Int:      return QMetaType::fromType<QList<int>>();
+        case QV4::CompiledData::CommonType::Bool:     return QMetaType::fromType<QList<bool>>();
+        case QV4::CompiledData::CommonType::Real:     return QMetaType::fromType<QList<qreal>>();
+        case QV4::CompiledData::CommonType::String:   return QMetaType::fromType<QList<QString>>();
+        case QV4::CompiledData::CommonType::Url:      return QMetaType::fromType<QList<QUrl>>();
+        case QV4::CompiledData::CommonType::Time:     return QMetaType::fromType<QList<QTime>>();
+        case QV4::CompiledData::CommonType::Date:     return QMetaType::fromType<QList<QDate>>();
+        case QV4::CompiledData::CommonType::DateTime: return QMetaType::fromType<QList<QDateTime>>();
+#if QT_CONFIG(regularexpression)
+        case QV4::CompiledData::CommonType::RegExp:   return QMetaType::fromType<QList<QRegularExpression>>();
+#else
+        case QV4::CompiledData::CommonType::RegExp:   return QMetaType();
+#endif
+        case QV4::CompiledData::CommonType::Rect:     return QMetaType::fromType<QList<QRectF>>();
+        case QV4::CompiledData::CommonType::Point:    return QMetaType::fromType<QList<QPointF>>();
+        case QV4::CompiledData::CommonType::Size:     return QMetaType::fromType<QList<QSizeF>>();
+        case QV4::CompiledData::CommonType::Invalid:  break;
+        };
+        return QMetaType {};
+    }
+
+    static QV4::CompiledData::CommonType propertyTypeForMetaType(QMetaType metaType) {
+        if (!metaType.isValid())
+            return QV4::CompiledData::CommonType::Void;
+
+        switch (metaType.id()) {
+        case QMetaType::QVariant:
+            return QV4::CompiledData::CommonType::Var;
+        case QMetaType::Int:
+            return QV4::CompiledData::CommonType::Int;
+        case QMetaType::Bool:
+            return QV4::CompiledData::CommonType::Bool;
+        case QMetaType::QReal:
+            return QV4::CompiledData::CommonType::Real;
+        case QMetaType::QString:
+            return QV4::CompiledData::CommonType::String;
+        case QMetaType::QUrl:
+            return QV4::CompiledData::CommonType::Url;
+        case QMetaType::QTime:
+            return QV4::CompiledData::CommonType::Time;
+        case QMetaType::QDate:
+            return QV4::CompiledData::CommonType::Date;
+        case QMetaType::QDateTime:
+            return QV4::CompiledData::CommonType::DateTime;
+#if QT_CONFIG(regularexpression)
+        case QMetaType::QRegularExpression:
+            return QV4::CompiledData::CommonType::RegExp;
+#endif
+        case QMetaType::QRectF:
+            return QV4::CompiledData::CommonType::Rect;
+        case QMetaType::QPointF:
+            return QV4::CompiledData::CommonType::Point;
+        case QMetaType::QSizeF:
+            return QV4::CompiledData::CommonType::Size;
+        default:
+            break;
+        }
+        return QV4::CompiledData::CommonType::Invalid;
+    }
+
+    static bool canCreateClassNameTypeByUrl(const QUrl &url);
     static QByteArray createClassNameTypeByUrl(const QUrl &url);
 
-    static QByteArray createClassNameForInlineComponent(const QUrl &baseUrl, int icId);
+    static QByteArray createClassNameForInlineComponent(const QUrl &baseUrl, const QString &name);
 
     struct IncrementalResult {
         // valid if and only if an error occurred
@@ -157,7 +255,7 @@ inline QQmlPropertyCacheCreator<ObjectContainer>::QQmlPropertyCacheCreator(QQmlP
     , typeClassName(typeClassName)
     , currentRoot(-1)
 {
-    propertyCaches->resize(objectContainer->objectCount());
+    propertyCaches->resetAndResize(objectContainer->objectCount());
 
     using namespace icutils;
 
@@ -233,8 +331,10 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::buildMetaObjectRecur
     };
 
     const CompiledObject *obj = objectContainer->objectAt(objectIndex);
-    bool needVMEMetaObject = isVMERequired == VMEMetaObjectIsRequired::Always || obj->propertyCount() != 0 || obj->aliasCount() != 0
-            || obj->signalCount() != 0 || obj->functionCount() != 0 || obj->enumCount() != 0
+    bool needVMEMetaObject = isVMERequired == VMEMetaObjectIsRequired::Always
+            || obj->propertyCount() != 0 || obj->aliasCount() != 0 || obj->signalCount() != 0
+            || obj->functionCount() != 0 || obj->enumCount() != 0
+            || obj->inlineComponentCount() != 0
             || ((obj->hasFlag(QV4::CompiledData::Object::IsComponent)
                  || (objectIndex == 0 && isAddressable(objectContainer->url())))
                 && !objectContainer->resolvedType(obj->inheritedTypeNameIndex)->isFullyDynamicType());
@@ -362,8 +462,9 @@ inline QQmlPropertyCache::ConstPtr QQmlPropertyCacheCreator<ObjectContainer>::pr
             Q_ASSERT(typeRef);
             QQmlType qmltype = typeRef->type();
             if (!qmltype.isValid()) {
-                imports->resolveType(stringAt(binding->propertyNameIndex),
-                                     &qmltype, nullptr, nullptr, nullptr);
+                imports->resolveType(
+                        QQmlTypeLoader::get(enginePrivate), stringAt(binding->propertyNameIndex),
+                        &qmltype, nullptr, nullptr);
             }
 
             const QMetaObject *attachedMo = qmltype.attachedPropertiesType(enginePrivate);
@@ -372,23 +473,6 @@ inline QQmlPropertyCache::ConstPtr QQmlPropertyCacheCreator<ObjectContainer>::pr
                 return nullptr;
             }
             return QQmlMetaType::propertyCache(attachedMo);
-        } else if (binding->isGroupProperty()) {
-            const auto *obj = objectContainer->objectAt(binding->value.objectIndex);
-            if (!stringAt(obj->inheritedTypeNameIndex).isEmpty())
-                return nullptr;
-
-            for (int i = 0, end = objectContainer->objectCount(); i != end; ++i) {
-                const auto *ext = objectContainer->objectAt(i);
-                if (ext->idNameIndex != binding->propertyNameIndex)
-                    continue;
-
-                if (ext->inheritedTypeNameIndex == 0)
-                    return nullptr;
-
-                QQmlBindingInstantiationContext pendingContext(i, &(*binding), QString(), nullptr);
-                pendingGroupPropertyBindings->append(pendingContext);
-                return nullptr;
-            }
         }
     }
     return nullptr;
@@ -459,8 +543,13 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
     // For property change signal override detection.
     // We prepopulate a set of signal names which already exist in the object,
     // and throw an error if there is a signal/method defined as an override.
-    QSet<QString> seenSignals;
-    seenSignals << QStringLiteral("destroyed") << QStringLiteral("parentChanged") << QStringLiteral("objectNameChanged");
+    // TODO: Remove AllowOverride once we can. No override should be allowed.
+    enum class AllowOverride { No, Yes };
+    QHash<QString, AllowOverride> seenSignals {
+        { QStringLiteral("destroyed"), AllowOverride::No },
+        { QStringLiteral("parentChanged"), AllowOverride::No },
+        { QStringLiteral("objectNameChanged"), AllowOverride::No }
+    };
     const QQmlPropertyCache *parentCache = cache.data();
     while ((parentCache = parentCache->parent().data())) {
         if (int pSigCount = parentCache->signalCount()) {
@@ -471,7 +560,15 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
                 for (QQmlPropertyCache::StringCache::ConstIterator iter = parentCache->stringCache.begin();
                      iter != parentCache->stringCache.end(); ++iter) {
                     if (currPSig == (*iter).second) {
-                        seenSignals.insert(iter.key());
+                        if (currPSig->isOverridableSignal()) {
+                            const qsizetype oldSize = seenSignals.size();
+                            AllowOverride &entry = seenSignals[iter.key()];
+                            if (seenSignals.size() != oldSize)
+                                entry = AllowOverride::Yes;
+                        } else {
+                            seenSignals[iter.key()] = AllowOverride::No;
+                        }
+
                         break;
                     }
                 }
@@ -485,8 +582,9 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
     for (  ; p != pend; ++p) {
         auto flags = QQmlPropertyData::defaultSignalFlags();
 
-        QString changedSigName = stringAt(p->nameIndex) + QLatin1String("Changed");
-        seenSignals.insert(changedSigName);
+        const QString changedSigName =
+                QQmlSignalNames::propertyNameToChangedSignalName(stringAt(p->nameIndex));
+        seenSignals[changedSigName] = AllowOverride::No;
 
         cache->appendSignal(changedSigName, flags, effectiveMethodIndex++);
     }
@@ -496,8 +594,9 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
     for ( ; a != aend; ++a) {
         auto flags = QQmlPropertyData::defaultSignalFlags();
 
-        QString changedSigName = stringAt(a->nameIndex()) + QLatin1String("Changed");
-        seenSignals.insert(changedSigName);
+        const QString changedSigName =
+                QQmlSignalNames::propertyNameToChangedSignalName(stringAt(a->nameIndex()));
+        seenSignals[changedSigName] = AllowOverride::No;
 
         cache->appendSignal(changedSigName, flags, effectiveMethodIndex++);
     }
@@ -549,10 +648,26 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
             flags.setHasArguments(true);
 
         QString signalName = stringAt(s->nameIndex);
-        if (seenSignals.contains(signalName))
-            return qQmlCompileError(s->location, QQmlPropertyCacheCreatorBase::tr("Duplicate signal name: invalid override of property change signal or superclass signal"));
-        seenSignals.insert(signalName);
-
+        const auto it = seenSignals.find(signalName);
+        if (it == seenSignals.end()) {
+            seenSignals[signalName] = AllowOverride::No;
+        } else {
+            // TODO: Remove the AllowOverride::Yes branch once we can.
+            QQmlError message = qQmlCompileError(
+                    s->location,
+                    QQmlPropertyCacheCreatorBase::tr(
+                            "Duplicate signal name: "
+                            "invalid override of property change signal or superclass signal"));
+            switch (*it) {
+            case AllowOverride::No:
+                return message;
+            case AllowOverride::Yes:
+                message.setUrl(objectContainer->url());
+                qCWarning(invalidOverride).noquote() << message.toString();
+                *it = AllowOverride::No; // No further overriding allowed.
+                break;
+            }
+        }
         cache->appendSignal(signalName, flags, effectiveMethodIndex++,
                             paramCount?paramTypes.constData():nullptr, names);
     }
@@ -565,8 +680,23 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
         auto flags = QQmlPropertyData::defaultSlotFlags();
 
         const QString slotName = stringAt(function->nameIndex);
-        if (seenSignals.contains(slotName))
-            return qQmlCompileError(function->location, QQmlPropertyCacheCreatorBase::tr("Duplicate method name: invalid override of property change signal or superclass signal"));
+        const auto it = seenSignals.constFind(slotName);
+        if (it != seenSignals.constEnd()) {
+            // TODO: Remove the AllowOverride::Yes branch once we can.
+            QQmlError message = qQmlCompileError(
+                function->location,
+                QQmlPropertyCacheCreatorBase::tr(
+                        "Duplicate method name: "
+                        "invalid override of property change signal or superclass signal"));
+            switch (*it) {
+            case AllowOverride::No:
+                return message;
+            case AllowOverride::Yes:
+                message.setUrl(objectContainer->url());
+                qCWarning(invalidOverride).noquote() << message.toString();
+                break;
+            }
+        }
         // Note: we don't append slotName to the seenSignals list, since we don't
         // protect against overriding change signals or methods with properties.
 
@@ -604,9 +734,9 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
         const QV4::CompiledData::CommonType type = p->commonType();
 
         if (p->isList())
-            propertyFlags.type = QQmlPropertyData::Flags::QListType;
+            propertyFlags.setType(QQmlPropertyData::Flags::QListType);
         else if (type == QV4::CompiledData::CommonType::Var)
-            propertyFlags.type = QQmlPropertyData::Flags::VarPropertyType;
+            propertyFlags.setType(QQmlPropertyData::Flags::VarPropertyType);
 
         if (type != QV4::CompiledData::CommonType::Invalid) {
             propertyType = p->isList()
@@ -618,43 +748,36 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
             QQmlType qmltype;
             bool selfReference = false;
             if (!imports->resolveType(
-                    stringAt(p->commonTypeOrTypeNameIndex()), &qmltype, nullptr, nullptr,
-                    nullptr, QQmlType::AnyRegistrationType, &selfReference)) {
+                        QQmlTypeLoader::get(enginePrivate),
+                        stringAt(p->commonTypeOrTypeNameIndex()), &qmltype, nullptr, nullptr,
+                        nullptr, QQmlType::AnyRegistrationType, &selfReference)) {
                 return qQmlCompileError(p->location, QQmlPropertyCacheCreatorBase::tr("Invalid property type"));
             }
 
             // inline components are not necessarily valid yet
-            Q_ASSERT(qmltype.isValid() || qmltype.isInlineComponentType());
+            Q_ASSERT(qmltype.isValid());
             if (qmltype.isComposite() || qmltype.isInlineComponentType()) {
-                CompositeMetaTypeIds typeIds;
+                QQmlType compositeType;
                 if (qmltype.isInlineComponentType()) {
-                    const QString icName = qmltype.elementName();
-                    auto containingType = qmltype.containingType();
-                    if (containingType.isValid()) {
-                        const QQmlType icType
-                            = QQmlMetaType::inlineComponentType(containingType, icName);
-                        typeIds = {icType.typeId(), icType.qListTypeId()};
-                    } else {
-                        typeIds = {};
-                    }
-                    if (!typeIds.isValid()) // type has not been registered yet, we must be in containing type
-                        typeIds = objectContainer->typeIdsForComponent(icName);
-                    Q_ASSERT(typeIds.isValid());
+                    compositeType = qmltype;
+                    Q_ASSERT(compositeType.isValid());
                 } else if (selfReference) {
-                     typeIds = objectContainer->typeIdsForComponent();
+                    compositeType = objectContainer->qmlTypeForComponent();
                 } else {
-                    QQmlRefPointer<QQmlTypeData> tdata = enginePrivate->typeLoader.getType(qmltype.sourceUrl());
+                    // compositeType may not be the same type as qmlType because multiple engines
+                    // may load different types for the same document. Therefore we have to ask
+                    // our engine's type loader here.
+                    QQmlRefPointer<QQmlTypeData> tdata
+                            = enginePrivate->typeLoader.getType(qmltype.sourceUrl());
                     Q_ASSERT(tdata);
                     Q_ASSERT(tdata->isComplete());
-
-                    auto compilationUnit = tdata->compilationUnit();
-                    typeIds = compilationUnit->typeIdsForComponent();
+                    compositeType = tdata->compilationUnit()->qmlTypeForComponent();
                 }
 
                 if (p->isList()) {
-                    propertyType = typeIds.listId;
+                    propertyType = compositeType.qListTypeId();
                 } else {
-                    propertyType = typeIds.id;
+                    propertyType = compositeType.typeId();
                 }
             } else {
                 if (p->isList())
@@ -665,11 +788,9 @@ inline QQmlError QQmlPropertyCacheCreator<ObjectContainer>::createMetaObject(
             }
 
             if (p->isList())
-                propertyFlags.type = QQmlPropertyData::Flags::QListType;
+                propertyFlags.setType(QQmlPropertyData::Flags::QListType);
             else if (propertyType.flags().testFlag(QMetaType::PointerToQObject))
-                propertyFlags.type = QQmlPropertyData::Flags::QObjectDerivedType;
-            else
-                propertyFlags.type = QQmlPropertyData::Flags::ValueType;
+                propertyFlags.setType(QQmlPropertyData::Flags::QObjectDerivedType);
         }
 
         if (!p->isReadOnly() && !propertyType.flags().testFlag(QMetaType::IsQmlList))
@@ -707,32 +828,27 @@ inline QMetaType QQmlPropertyCacheCreator<ObjectContainer>::metaTypeForParameter
         *customTypeName = typeName;
     QQmlType qmltype;
     bool selfReference = false;
-    if (!imports->resolveType(typeName, &qmltype, nullptr, nullptr, nullptr,
-                              QQmlType::AnyRegistrationType, &selfReference))
+    if (!imports->resolveType(
+                &enginePrivate->typeLoader, typeName, &qmltype, nullptr, nullptr, nullptr,
+                QQmlType::AnyRegistrationType, &selfReference))
         return QMetaType();
 
     if (!qmltype.isComposite()) {
         const QMetaType typeId = param.isList() ? qmltype.qListTypeId() : qmltype.typeId();
         if (!typeId.isValid() && qmltype.isInlineComponentType()) {
-            const auto typeIds = objectContainer->typeIdsForComponent(qmltype.elementName());
-            return param.isList() ? typeIds.listId : typeIds.id;
+            const QQmlType qmlType = objectContainer->qmlTypeForComponent(qmltype.elementName());
+            return param.isList() ? qmlType.qListTypeId() : qmlType.typeId();
         } else {
             return typeId;
         }
     }
 
     if (selfReference) {
-        const auto typeIds = objectContainer->typeIdsForComponent();
-        return param.isList() ? typeIds.listId : typeIds.id;
+        const QQmlType qmlType = objectContainer->qmlTypeForComponent();
+        return param.isList() ? qmlType.qListTypeId() : qmlType.typeId();
     }
 
-    QQmlRefPointer<QQmlTypeData> tdata = enginePrivate->typeLoader.getType(qmltype.sourceUrl());
-    Q_ASSERT(tdata);
-    Q_ASSERT(tdata->isComplete());
-
-    auto compilationUnit = tdata->compilationUnit();
-
-    return param.isList() ? compilationUnit->typeIds.listId : compilationUnit->typeIds.id;
+    return param.isList() ? qmltype.qListTypeId() : qmltype.typeId();
 }
 
 template <typename ObjectContainer, typename CompiledObject>
@@ -785,7 +901,7 @@ inline QQmlError QQmlPropertyCacheAliasCreator<ObjectContainer>::propertyDataFor
     *type = QMetaType();
     bool writable = false;
     bool resettable = false;
-    bool bindable = false;
+    bool notifiesViaBindable = false;
 
     propertyFlags->setIsAlias(true);
 
@@ -837,16 +953,16 @@ inline QQmlError QQmlPropertyCacheAliasCreator<ObjectContainer>::propertyDataFor
         if (referencedType.isValid()) {
             *type = referencedType.typeId();
             if (!type->isValid() && referencedType.isInlineComponentType()) {
-                *type = objectContainer->typeIdsForComponent(referencedType.elementName()).id;
+                *type = objectContainer->qmlTypeForComponent(referencedType.elementName()).typeId();
                 Q_ASSERT(type->isValid());
             }
         } else {
-            *type = typeRef->compilationUnit()->typeIds.id;
+            *type = typeRef->compilationUnit()->metaType();
         }
 
         *version = typeRef->version();
 
-        propertyFlags->type = QQmlPropertyData::Flags::QObjectDerivedType;
+        propertyFlags->setType(QQmlPropertyData::Flags::QObjectDerivedType);
     } else {
         int coreIndex = QQmlPropertyIndex::fromEncoded(alias.encodedMetaPropertyIndex).coreIndex();
         int valueTypeIndex = QQmlPropertyIndex::fromEncoded(
@@ -864,12 +980,12 @@ inline QQmlError QQmlPropertyCacheAliasCreator<ObjectContainer>::propertyDataFor
             *type = property->propType();
             writable = property->isWritable();
             resettable = property->isResettable();
-            bindable = property->isBindable();
+            notifiesViaBindable = property->notifiesViaBindable();
 
-            // Copy type flags
-            propertyFlags->copyPropertyTypeFlags(property->flags());
             if (property->isVarProperty())
-                propertyFlags->type = QQmlPropertyData::Flags::QVariantType;
+                propertyFlags->setType(QQmlPropertyData::Flags::QVariantType);
+            else
+                propertyFlags->copyPropertyTypeFlags(property->flags());
         };
 
         // for deep aliases, valueTypeIndex is always set
@@ -912,7 +1028,8 @@ inline QQmlError QQmlPropertyCacheAliasCreator<ObjectContainer>::propertyDataFor
                 resettable = writable && valueTypeMetaProperty.isResettable();
                 writable = writable && valueTypeMetaProperty.isWritable();
 
-                bindable = valueTypeMetaProperty.isBindable();
+                // Do not update notifiesViaBindable. The core property counts for notifications.
+                propertyFlags->setIsDeepAlias(true);
             }
         }
     }
@@ -920,7 +1037,7 @@ inline QQmlError QQmlPropertyCacheAliasCreator<ObjectContainer>::propertyDataFor
     propertyFlags->setIsWritable(
             writable && !alias.hasFlag(QV4::CompiledData::Alias::IsReadOnly));
     propertyFlags->setIsResettable(resettable);
-    propertyFlags->setIsBindable(bindable);
+    propertyFlags->setIsBindable(notifiesViaBindable);
     return QQmlError();
 }
 
